@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "$Id: types.c,v 1.2 92/11/03 02:48:33 genek Exp $";
+static char rcsid[] = "$Id: types.c,v 1.4 1993/12/13 04:02:27 genek Exp $";
 #endif
 
 #include "../include/config.h"
@@ -10,6 +10,8 @@ static char rcsid[] = "$Id: types.c,v 1.2 92/11/03 02:48:33 genek Exp $";
 #include <stdlib.h>
 #endif
 
+void typeis();
+
 static void
 usage()
 {
@@ -20,6 +22,7 @@ usage()
 char *pc;
 int isunsigned;
 
+void
 main(argc, argv)
     int argc;
     char *argv[];
@@ -61,6 +64,7 @@ main(argc, argv)
 
 }
 
+void
 typeis(i)
     int i;
 {
@@ -68,11 +72,14 @@ typeis(i)
     int longlen = sizeof(unsigned long);
     char sign[64];
 
+#ifdef ATT_3B2 		/* ATT 3B2: Thanks Joe Sirrianni */
+    strcpy(sign, "unsigned");
+#else
     if (isunsigned)
 	strcpy(sign, "unsigned");
     else
 	strcpy(sign, "signed");
-
+#endif
 
     if (i == shortlen)
 	printf("typedef %s short %s;\n", sign, pc);
