@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "$Id: snefru.c,v 1.1.1.2 92/11/02 18:22:51 genek Exp $";
+static char rcsid[] = "$Id: snefru.c,v 1.6 1993/12/15 18:28:06 genek Exp $";
 #endif
 
 /*
@@ -19,7 +19,7 @@ static char rcsid[] = "$Id: snefru.c,v 1.1.1.2 92/11/02 18:22:51 genek Exp $";
  * September 27, 1992
  */
 
-char *ltob64();
+char *pltob64();
 
 /*
   This is an implementation of Snefru. Snefru is a one-way hash
@@ -235,9 +235,6 @@ Note that the test cases given are from 8-pass Snefru (e.g., SECURITY_LEVEL=8)
 				 * from the hash routine */
 #endif
 #define	CHUNK_SIZE (INPUT_BLOCK_SIZE - OUTPUT_BLOCK_SIZE)
-
-#define BUFSIZE (CHUNK_SIZE*512)		/* size of input file buffer */
-
 /* generate a mask with log-to-the-base-2(INPUT_BLOCK_SIZE) "1" bits  */
 #define MASK (INPUT_BLOCK_SIZE - 1)
 
@@ -251,7 +248,7 @@ Note that the test cases given are from 8-pass Snefru (e.g., SECURITY_LEVEL=8)
  * line option to the C compiler. 
  */
 #ifndef SECURITY_LEVEL
-#define SECURITY_LEVEL 4
+#define SECURITY_LEVEL 8
 #endif
 #define MAX_SBOX_COUNT 16
 
@@ -260,7 +257,7 @@ char ident[] ="@(#) Snefru Version 2.5a November 29, 1990";
 
 typedef unsigned long int word32;
 /* The standard S-Box  */
-static word32 standardSBoxes[MAX_SBOX_COUNT][256]= {
+word32 standardSBoxes[MAX_SBOX_COUNT][256]= {
 
  {  /* Start of S Box 0  */
 
@@ -1187,6 +1184,36 @@ ErrAbort (s)
 }
 
 
+/*
+ * The following routine converts a byte array to an array of word32
+ * int.  It is primarily intended to eliminate the byte-ordering problem.
+ * VAXes order the bytes in a character array differently than SUN's do. This
+ * routine should be portable across different computer architectures.
+ * However, it is not very efficient.
+ */
+void
+ConvertChunkOfBytes (charBuffer, wordBuffer)
+	/* an input buffer of characters */
+	char    charBuffer[CHUNK_SIZE*4];
+	/* an output buffer of word32's */
+	word32 wordBuffer[CHUNK_SIZE];
+{
+	int     i;
+	word32 t0, t1, t2, t3;
+
+	for (i = 0; i < CHUNK_SIZE; i++) {
+		t0 = charBuffer[4*i];
+		t1 = charBuffer[4*i + 1];
+		t2 = charBuffer[4*i + 2];
+		t3 = charBuffer[4*i + 3];
+		t0 &= 0xffL;
+		t1 &= 0xffL;
+		t2 &= 0xffL;
+		t3 &= 0xffL;
+		wordBuffer[i] = (t0 << 24) | (t1 << 16) | (t2 << 8) | t3;
+	};
+}
+
 /*
  * The following routine trys to read 4*"CHUNK_SIZE" bytes from the input
  * file. It will only return a non-full buffer if an EOF is encountered.
@@ -1196,15 +1223,15 @@ ErrAbort (s)
  * pads the output array, "buf", with trailing 0 bytes.
  */
 int
-ReadChunk (buf)
+ReadChunk (buf, fd)
 	word32 buf[CHUNK_SIZE];
+	int fd;
 {
 	char    charBuf[CHUNK_SIZE*4];
 	int     byteCount;
 	int     i;
-	unsigned long *pl = (unsigned long *) charBuf;
 
-	byteCount = fread (charBuf, 1, CHUNK_SIZE*4, stdin);
+	byteCount = read(fd, charBuf, CHUNK_SIZE*4);
 	if (byteCount < 0)
 			ErrAbort ("error on read");
 	/* Zero out rest of buffer  */
@@ -1213,21 +1240,86 @@ ReadChunk (buf)
 	/*
 	 * The following conversion is required for machines with
 	 * byte-ordering unlike the SUN's (68000's or SPARC)
+	 *   ConvertChunkOfBytes (charBuf, buf);
 	 */
-	/* Inline this 
-	    ConvertChunkOfBytes (charBuf, buf);
-	*/
+	{
+	    unsigned long *plsrc = (unsigned long *) charBuf,
+			  *pldest = (unsigned long *) buf;
 
-	for (i = 0; i < CHUNK_SIZE; i++) {
-	    buf[i] = ((unsigned long) *pl);
-	    ntohl(buf[i]);
-	    pl++;
+	    for (i = 0; i < CHUNK_SIZE; i++) {
+		pldest[i] = *plsrc++;
+		ntohl(pldest[i]);
+	    }
 	}
 
 	return (byteCount);
 }
 
 
+/*
+ * HashN is a conceptually simpler and more general version of Hash512.
+ * It accepts an input of
+ * INPUT_BLOCK_SIZE 32-bit words and produces an output of
+ * OUTPUT_BLOCK_SIZE 32-bit words.  The INPUT_BLOCK_SIZE must be at
+ * least two words larger than the output, or a serious degradation
+ * in security will occur.  In addition, the use of outputs less
+ * than 128 bits will often significantly reduce security.  While
+ * possible, it requires great caution.
+ */
+void
+HashN (output, input)
+	word32 output[OUTPUT_BLOCK_SIZE];
+	word32 input[INPUT_BLOCK_SIZE];
+{
+	static int shiftTable[4] = {16, 8, 16, 24};
+	/* the array of data being hashed  */
+	word32 block[INPUT_BLOCK_SIZE];
+	word32 SBoxEntry;	/* just a temporary */
+	int     shift;
+	int     i;
+	int     index;
+	int     next, last;
+	int     byteInWord;
+
+	/* initialize the block to be encrypted from the input  */
+	for (i = 0; i < INPUT_BLOCK_SIZE; i++)
+		block[i] = input[i];
+
+	for (index = 0; index < SECURITY_LEVEL; index++) {
+		for (byteInWord = 0; byteInWord < 4; byteInWord++) {
+			for (i = 0; i < INPUT_BLOCK_SIZE; i++) {
+				/* compute i+1 and i-1 mod INPUT_BLOCK_SIZE */
+				next = (i + 1) & MASK;
+				last = (i + MASK) & MASK;
+				/* Load an entry from the S-box  */
+				SBoxEntry = standardSBoxes
+					[2*index + ((i / 2) & 1)]
+					[block[i] & 0xffL];
+				/*
+				 * and XOR that entry with the preceding and
+				 * following words
+				 */
+				block[next] ^= SBoxEntry;
+				block[last] ^= SBoxEntry;
+			};
+			/*
+			 * Rotate right all 32-bit words in the entire block
+			 * at once.
+			 */
+			shift = shiftTable[byteInWord];
+			for (i = 0; i < INPUT_BLOCK_SIZE; i++)
+				block[i] = (block[i] >> shift) |
+					(block[i] << (32 - shift));
+
+		};		/* end of byteInWord going from 0 to 3 */
+	};			/* end of index going from 0 to
+				 * SECURITY_LEVEL-1 */
+
+	for (i = 0; i < OUTPUT_BLOCK_SIZE; i++)
+		output[i] = input[i] ^ block[MASK - i];
+}
+
+
 /*
  * Hash512 is a more efficient and specialized version of HashN.
  * It accepts an input of INPUT_BLOCK_SIZE 32-bit words and
@@ -1245,10 +1337,12 @@ Hash512 (output, input)
 {
 	static int shiftTable[4] = {16, 8, 16, 24};
 	/* the array of data being hashed  */
+	word32	block[INPUT_BLOCK_SIZE];
 	word32	SBE;	/* just a temporary */
 	int     shift, leftShift;
 	int     i;
 	int     index;
+	int     next, last;
 	int     byteInWord;
 	word32	*SBox0;
 	word32	*SBox1;
@@ -1335,7 +1429,7 @@ Hash512 (output, input)
 /*
  * This routine increments a 64-bit counter by the given increment.
  */
-static void
+void
 Increment64BitCounter (counter, increment)
 	word32 counter[2];
 	long int increment;
@@ -1385,16 +1479,6 @@ Increment64BitCounter (counter, increment)
  * array is left justified and zero-filled on the right.
  * 
  */
-
-
-/*
- * sig_snefru_get(int fd_int, char *ps_signature, int siglen) 
- *
- *	fd_in:		pointer to input file descriptor
- *	ps_signature:	pointer to array where signature will be stored
- *	siglen:		length of the signature array (for overflow checking)
- */
-
 int
 sig_snefru_get(fd_in, ps_signature, siglen) 
     int fd_in; 
@@ -1402,24 +1486,21 @@ sig_snefru_get(fd_in, ps_signature, siglen)
     int siglen;
 {
 	int     i;
+	char s[512], sword[512];
 	word32 hash[INPUT_BLOCK_SIZE];
 	word32 bitCount[2];	/* the 64-bit count of the number of
 					 * bits in the input */
-	unsigned char buffer[BUFSIZE];
-	unsigned long *plsrc, *pldest;
-	int		readin = -1;
-	char		s[256];
-	char		sword[128];
-
-    ps_signature[0] = '\0';
-
-    /* rewind the file descriptor */
-    if (lseek(fd_in, 0, SEEK_SET) < 0) {
-	perror("sig_snefru_get: lseek()");
-	exit(1);
-    }
+	long int byteCount;	/* the count of the number of bytes we just
+				 * read */
 
+	ps_signature[0] = 0;
+	/* rewind the file descriptor */
+	if (lseek(fd_in, 0, SEEK_SET) < 0) {
+	    perror("sig_snefru_get: lseek()");
+	    exit(1);
+	}
 
+#ifdef MAIN
 	/* Test for various error conditions and logic problems  */
 	if (SECURITY_LEVEL*2 > MAX_SBOX_COUNT)
 		ErrAbort ("Too few S-boxes for specified SECURITY_LEVEL");
@@ -1433,6 +1514,7 @@ sig_snefru_get(fd_in, ps_signature, siglen)
 		ErrAbort ("logic error, OUTPUT_BLOCK_SIZE is too big");
 	if (CHUNK_SIZE < 2)
 		ErrAbort ("logic error, CHUNK_SIZE is too small");
+#endif
 
 	/*
 	 * The error condtions have now been checked -- everything should
@@ -1448,42 +1530,14 @@ sig_snefru_get(fd_in, ps_signature, siglen)
 	 * (32) bytes of hash holds the output of the previous hash
 	 * computation done during the previous iteration of the loop
 	 */
-    pldest = (unsigned long *) &hash[OUTPUT_BLOCK_SIZE];
-    while ((readin = read(fd_in, (char *)buffer, (off_t) BUFSIZE)) > 0) {
-
-	Increment64BitCounter (bitCount, (long)(8*readin));
-
-	/* Zero out rest of buffer -- usually drops out */
-	for (i = readin; i < BUFSIZE; i++) {
-	    buffer[i] = '\0';
-	}
-	plsrc = (unsigned long *) buffer;
-
-	while (readin > 0) {
-	    /* copy into array of longs */
-	    for (i = 0; i < CHUNK_SIZE; i++) {
-		pldest[i] = ((unsigned long) *plsrc);
-		ntohl(pldest[i]);
-		plsrc++;
-	    }
-#ifdef DEBUG
-	    for (i = 0; i < OUTPUT_BLOCK_SIZE; i++)
-		    printf ("-%08lx", hash[i]);
-	    printf("\n");
-#endif
-	    Hash512(hash, hash);
-	    readin -= CHUNK_SIZE * 4;
-#ifdef DEBUG
-	    for (i = 0; i < OUTPUT_BLOCK_SIZE; i++)
-		    printf (" %08lx", hash[i]);
-	    printf("\n");
-#endif
-	}
-    }
-    /* Zero out the rest of the hash array */
-    for (i = 0; i < CHUNK_SIZE; i++) {
-	pldest[i] = 0L;
-    }
+	do {
+		/* Get the next chunk */
+		byteCount = ReadChunk (&hash[OUTPUT_BLOCK_SIZE], fd_in);
+		Increment64BitCounter (bitCount, 8*byteCount);
+		/* hash in the block we just read  */
+		if (byteCount > 0)
+			Hash512 (hash, hash);
+	} while (byteCount > 0);  /* end of while */
 
 	/*
 	 * Put the 64-bit bit-count into the final 64-bits of the block about
@@ -1500,14 +1554,23 @@ sig_snefru_get(fd_in, ps_signature, siglen)
 	 * the first OUTPUT_BLOCK_SIZE words of "hash" now hold the hashed
 	 * result, which is printed on standard output
 	 */
-	for (i = 0; i < OUTPUT_BLOCK_SIZE; i++) {
-		if (printhex)
-		    sprintf (s, "%08lx", hash[i]);
-		else
-		    sprintf (s, "%6s", ltob64(hash[i], sword));
+	/*
+	 * the first OUTPUT_BLOCK_SIZE words of "hash" now hold the hashed
+	 * result, which is printed on standard output
+	 */
+	if (printhex) {
+	    for (i = 0; i < OUTPUT_BLOCK_SIZE; i++) {
+		sprintf (s, "%08lx", hash[i]);
 		strcat(ps_signature, s);
+	    }
+	}
+	/* base 64 */
+	else {
+	    pltob64(hash, ps_signature, OUTPUT_BLOCK_SIZE);
 	}
 	/*
+	for (i = 0; i < OUTPUT_BLOCK_SIZE; i++)
+		printf (" %08lx", hash[i]);
 	 * Note that if the user wishes to view the output as a sequence
 	 * of bytes, then the byte order is the same as the order
 	 * in which the bytes are printed by "printf" above.  The
@@ -1540,6 +1603,8 @@ sig_snefru_get(fd_in, ps_signature, siglen)
          *
 	 * When the output size is other than 128 bits, the above comments
 	 * should be extended in the obvious fashion.
+	printf ("\n");
+	exit (0);
 	 */
 	return 0;
 }
